/*
 * PlanJBounce components are Open Source
 * PlanJBounce requires PlanJCore (not Open Source)

 * Http view of Bouncer Stats 2023-09-23
 * Interface V2 = /BView 2023-10-01
2023-10-03 Minor adjustments, will require Date based filtering for long runs
   Have to plan a 'Time' Filter
   Country Filter (last part of Origin), group by Origin (pre - or space), forwarded Host reject type, 
2023-10-10 Filtering implemented, first variant, one parameter at a time
2023-10-11 Adding /BNode - returns full json data - /BNode?purge = Flush Data, buttons on main page
2023-10-13 Origin modified to include Country and Owner / modify filters etc
2023-10-15 Filtering adjustments
2023-10-17 Filtering Crash, null Country, fix Host parse in Good
2023-10-18 adapt to IPOrigin
2023-10-22 Text and Configuration changes
2023-10-23 BView text and cosmetic adjustments, 0.6.2 UserAgent Display in connect section (temporary)
2023-10-24 Confirmed BNode require byte[] (+2bytes vs String) Stable 40h, 100 IP, 400 hits
 */
package ca.planj.bounce;

import ca.planj.data.Node;
import ca.planj.http.httpModule;

/**
 * BounceView is a PlanJCore httpModule to display Bouncer status and data from /BView URI
 * @author jipi@planj.ca
 */
public class BounceView implements httpModule {
    public static String Version = "PlanJ BounceView 0.6.3 2023-10-25";
    public static boolean debug = false;
    
    /**
     * Plain httpModule single Instance selection
     * @return 
     */
    public httpModule getInstance() {
        return this;
    }
    
    /**
     * Currently no configuration requirements
     * @return 
     */
    public Node dNodeParams() {
        Node D = Node.makeDataNode(getClass().getCanonicalName()); 
        D.addChild(Node.makeDataNode("Type", "httpM"));
        return D;
    }
    
    /**
     * Standard PlanJCore httpModule processing zone, respond to /BView URI
     * @param H  Parent httpServer managing session and connection data
     * @return 
     */
    public boolean process(ca.planj.com.handle.httpServer H) {
        if (H.URI.startsWith("/BNode")) {
            String R = "NoOp";
            String CT = "text/plain";
            if (H.Params.get("purge") != null) {
                Bouncer.Flush();
                R = "Flushing all Bouncer Clients";
            } else {
                ca.planj.data.Node N = ca.planj.data.Node.makeDataNode("PlanJBouncerData");
                String[] IPs = ca.planj.util.SortBox.sortKeys(Bouncer.Clients);
                for (int i=0; i<IPs.length; i++) try {
                    Bouncer B = Bouncer.getFor(IPs[i]);
                    N.addChild(B.getData());
                } catch (Exception y) {}
                R = ca.planj.data.nodeio.JSonIO.getInstance().format(N) + "\n";
                CT = "application/json";
            }
            if (R != null) {
                byte[] K = R.getBytes();
                // byte[] fait la différence 2023-10-24 17:06:15 INFO null BView Anomaly9390 9388
                // if (K.length != R.length()) H.Link.logErr("BView Anomaly" + K.length + " " + R.length());
                H.Link.write(H.getHeader(200, K.length, CT, null));
                H.Link.write(K);
                if (debug) System.out.println("Sent " + H.Link);
                return true;
            }
            
        } else if (H.URI.startsWith("/BView")) {
            long t = System.currentTimeMillis();
            String R = "<html><head><title>Bouncer Stats</title></head><style>body {\n  font-family: sans-serif;\n  font-size: 14px;\n}\n"
                    + ".bt {\n  font-family: sans-serif;\n  font-size: 12px;\n}\n"
                    + "table {\n  border: 1px solid black;\n  border-collapse: collapse;\n}\n"
                    + "th, td {\n  border: 1px solid black;\n  font-family: sans-serif;\n  font-size: 12px;\n}\n"
                    + "</style><script>\n";
            //document.location='getUR() + \"?Pings\"
            R += "function getPing() {\n document.location=getUR() + '?Pings';  \n}\n";
            R += "function getUR() {\n var UR = document.URL;\n if (UR.indexOf('?') > -1) UR = UR.substring(0,UR.indexOf('?'));\n return UR;  \n}\n";
            R += "function setFilt(cbo) {\n document.location=getUR() + '?' + cbo.name + '=' + cbo.value;  \n}\n";
            R += "</script>\n<body>\n";
            R += "<b>" + Version + " -  refreshed on " + ca.planj.util.ISODate.getISODate() + "</b>";
            R += "<br>IP Origin Resolver : " + ca.planj.ip.util.IPOrigin.getVersion() + "<br><br>";
            
            // DataBlock
            String[] IPs = ca.planj.util.SortBox.sortKeys(Bouncer.Clients);
            if (IPs == null) R += "<hr><center>*** No Data ***</center><hr>";
            else if (IPs.length < 1) R += "<hr><center>*** Empty Data ***</center><hr>";
            else {
                // Prep Filters - read from Params ?Country=CN&Origin=CENSY&Host=planj.ca&Reason=Executable
                String M = "-";
                java.util.Hashtable Countries = new java.util.Hashtable(15);
                java.util.Hashtable Origins = new java.util.Hashtable(15);
                java.util.Hashtable Hosts = new java.util.Hashtable(15);
                java.util.Hashtable Reasons = new java.util.Hashtable(15);
                for (int i=0; i<IPs.length; i++) try {
                    Bouncer B = Bouncer.getFor(IPs[i]);
                    String[] O = B.getOrigin();
                    if (O[1] != null) Countries.put(O[1],M);
                    if (O[0] != null) Origins.put(O[0], M);
                    if (debug) 
                        System.out.println("Adding " + O[0] + " - " + O[1]);
                    for (int j=0; j<B.Good.size(); j++) try {
                        if (debug) System.out.print("GoodL " + j + " ");
                        String T = (String) B.Good.elementAt(j);
                        // 2023-10-17 10:49:03 planj.ca /
                        T = T.substring(20,T.indexOf(" ",21)).trim();
                        //if (T.contains(" ")) T = T.substring(0,T.indexOf(" "));
                        if (debug) 
                            System.out.println("Adding " + T);
                        Hosts.put(T,M);
                    } catch (Exception good) {}
                    for (int j=0; j<B.Bad.size(); j++) try {
                        if (debug) System.out.print("BadL " + j + " ");
                        String T = (String) B.Bad.elementAt(j);
                        T = T.substring(20,T.indexOf(" :",21));
                        if (debug)
                            System.out.println("Adding " + T);
                        Reasons.put(T,M);
                    } catch (Exception Bad) {}
                } catch (Exception S) {
                    ca.planj.log.Logger.log("Error","BounceView","PrepFilter " + IPs[i] + " " + S + " " + S.getStackTrace()[1]);
                }
                // Display Filters
                R += "<hr>Filters : ";
                String Orig = (String) H.Params.get("Origin");
                if (Orig == null) Orig = "";
                String Coun = (String) H.Params.get("Country");
                if (Coun == null) Coun = "";
                String Reas = (String) H.Params.get("Reason");
                if (Reas == null) Reas = "";
                Reas = ca.planj.util.EncodeMgr.UnEscape(Reas);
                String Hos = (String) H.Params.get("Host");
                if (Hos == null) Hos = "";
                boolean Pi = (H.Params.get("Pings") != null);
                if (debug) System.out.println("O " + Orig + "\nC " + Coun + "\nR " + Reas + "\nH " + Hos + "\nP " + Pi);
                
                String[] A = null;
                if (Countries != null) {
                    A = ca.planj.util.SortBox.sortKeys(Countries);
                    R += "Country : <select name='Country' onchange='setFilt(this)'><option>-</option>";
                    for (int i=0; i<A.length; i++) {
                        String Z = "";
                        if (Coun.length() > 0) if (Coun.equals(A[i])) Z = " selected";
                        R += "<option value='" + A[i] + "'" + Z + ">" + A[i] + "</option>";
                    }
                    R += "</select>\n";
                }
                if (Origins != null) {
                    A = ca.planj.util.SortBox.sortKeys(Origins);
                    R += " - Origin : <select name='Origin' onchange='setFilt(this)'><option>-</option>";
                    for (int i=0; i<A.length; i++) {
                        String Z = "";
                        if (Orig.length() > 0) if (Orig.equals(A[i])) Z = " selected";
                        R += "<option value='" + A[i] + "'" + Z + ">" + A[i] + "</option>";
                    }
                    R += "</select>\n";
                }
                if (Hosts != null) {
                    A = ca.planj.util.SortBox.sortKeys(Hosts);
                    R += " - Host : <select name='Host' onchange='setFilt(this)'><option>-</option>";
                    for (int i=0; i<A.length; i++) {
                        String Z = "";
                        if (Hos.length() > 0) if (Hos.equals(A[i])) Z = " selected";
                        R += "<option value='" + A[i] + "'" + Z + ">" + A[i] + "</option>";
                    }
                    R += "</select>\n";
                }
                if (Reasons != null) {
                    A = ca.planj.util.SortBox.sortKeys(Reasons);
                    R += " - Reason : <select name='Reason' onchange='setFilt(this)'><option>-</option>";
                    for (int i=0; i<A.length; i++) {
                        String Z = "";
                        if (Reas.length() > 0) if (Reas.equals(A[i])) Z = " selected";
                        R += "<option value='" + A[i] + "'" + Z + ">" + A[i] + "</option>";
                    }
                    R += "</select>\n";
                }
                R+= " - <input type='button' value='Pingers' onclick='getPing();' />\n - <input type='button' value='Full List' onclick='document.location=getUR();' />\n";
                R+= " - - - - - - - - - - - - - - - <input type='button' value='Purge DB' onclick='document.location=\"/BNode?purge\";' />\n - <input type='button' value='Get Data' onclick='document.location=\"/BNode\";' />\n";
                R += "<hr>";
                
                // Display Data
                R += "<table width='95%'><tr><th>IP / DNS / UA (" + IPs.length + ")</th><th>Repl / Hits</th><th>Stripped</th><th>Origin</th><th>Forwarded Requests</th><th>Rejected Requests</th></tr>\n";
                for (int i=0; i<IPs.length; i++) try {
                    Bouncer B = Bouncer.getFor(IPs[i]);
                    String[] O = B.getOrigin();
                    String Goods = "";
                    // Good
                    java.util.Vector V = B.filterGood();  // For full listing, directly use Bouncer.Good
                    for (int j=0; j<V.size(); j++) {
                        String S = (Goods.length()>0?"<br>":"") + V.elementAt(j); // Filter on Hos
                        if (Hos.length()>0) {
                            if (S.contains(Hos)) Goods += S;
                        } else Goods += S;
                    }
                    
                    String Bads = "";
                    // Bad
                    V = B.filterBad();  // For full listing, directly use Bouncer.Bad
                    for (int j=0; j<V.size(); j++) {
                        String S = (Bads.length()>0?"<br>":"") + V.elementAt(j); // Filter on Reas
                        if (Reas.length()>0){
                            if (S.contains(Reas)) Bads += S;
                        } else Bads += S;
                    }
                    boolean doDisp = (Orig.length()<1) && (Coun.length()<1) && (Reas.length()<1) && (Hos.length()<1);  // No filter, display all
                    if (Pi) if (B.Good.size() == 0) if (B.Bad.size() == 0) doDisp = true;
                    if (debug) System.out.println("Disp-0=" + doDisp);
                    if (!doDisp) {
                        String Z = O[1]; //.getOrigin();
                        if (Coun.length() > 0) doDisp = Z.equals(Coun);
                        if (!doDisp) if (Orig.length() > 0) doDisp = Orig.equals(O[0]);
                    }
                    if (debug) System.out.println("Disp-1=" + doDisp);
                    if (!doDisp) {
                        if (Hos.length()>0) if (Goods.length() > 0) doDisp = true;
                        if (Reas.length()>0) if (Bads.length() > 0) doDisp = true;
                    }   // filtered display limited
                    if (debug) System.out.println("Disp-2=" + doDisp);
                    if (doDisp) {
                        String OR = O[2];
                        if (OR == null) OR = "Unknown";
                        int sp = 0;
                        if ((sp = OR.indexOf(" ")) > 3) OR = OR.substring(0,sp) + "<br>" + OR.substring(sp+1);
                        String D = B.getDNS();
                        R += "<tr><td valign='top'>" + IPs[i] + (D.equals(IPs[i])?"":"<br>"+D) + (B.UserAgent != null?"<br>"+B.UserAgent:"") + "</td><td valign='top' align='center'>" + B.Logged + " / " + B.Count + 
                               
                                "</td><td valign='top' align='center'>" + B.Stripped + "</td><td valign='top'>" + OR + "</td><td valign='top'>" + Goods;
                        R += "</td><td valign='top'>" + Bads;
                        R += "</td></tr>\n";
                    }
                } catch (Exception z) {
                    ca.planj.log.Logger.log("Error","BounceView","Prepdata " + z + " " + z.getStackTrace()[0]);
                }
                R += "</table>";
            }
            
            // Informational Block
            if (debug) System.out.println("BounceView Built Response " + (System.currentTimeMillis()-t));
            R += "<br>End of Report<br>" + planjbounce.PlanJBounce.Version + " on " + ca.planj.PlanJCore.Version + " - &copy; 1996-2023 <a href='mailto:planj@planj.ca'>PlanJ Solutions inc.</a> <a href='https://planj.ca'>https://planj.ca</a>";
            R += "<br><br><table cellpadding='5'><tr><td valign='top'>";
            if (LLL < 0) LLL = ca.planj.util.StringUtils.ParseIt(planjbounce.PlanJBounce.Bounce, "\n").length + 1;
            R += "To customize, edit <b>" + ca.planj.core.monit.SysMonitor.getBootInfo().getValue("Path.AppCfg") + "</b><br>It is autogenerated if not present<br>Active Configuration<br>"
                    + "<textarea cols='100' rows='" + LLL + "' class='bt'>" + planjbounce.PlanJBounce.Bounce + "</textarea>";
            R += "<br>The default configuration has<br>- a Bouncer server on Port 8786 - where you can forward Internet port 80<br>- a Status Server with only /BView and /PJStatus on port 8787<br>"
                    + "The Status server loads an Access Control List (LAN_ACL) to only allow access from a Local Area Network</td><td valign='top'>";
            // Param Block
            R += "<b>Active Modes and configuration<br></b><br><table>";
            R += "<tr><td>Pattern</td><td colspan='4'>" + httpBounce.Prefix + "[Host Header Value]" + httpBounce.Suffix + " - can specify a fixed or host redirect and include a suffix</td></tr>";
            R += "<tr><td>Modes</td><td>Current</td><td>Default</td><td>Strict</td><td>specify restrictions (optional console output with Track and Debug modes)</td></tr>";
            R += "<tr><td>Polite</td><th>" + httpBounce.Polite + "</th><td>true</td><td>false</td><td>if true, sends a 404 response on GET and noProc, if false, just cuts off the connection</td></tr>";
            R += "<tr><td>Strip</td><th>" +  httpBounce.Strip + "</th><td>true</td><td>true</td><td>if true, removes all URL Parameter before redirecting URI?Param=Val - request is redirected with lower risk</td></tr>";
            R += "<tr><td>GET</td><th>" +    httpBounce.Get +    "</th><td>true</td><td>true</td><td>if true, only let GET Methods through, rejects POST, HEAD, PUT, DELETE etc</td></tr>";
            R += "<tr><td>noProc</td><th>" + httpBounce.noProc + "</th><td>true</td><td>true</td><td>if true, rejects all requests for .jsp, .asp(x), .php, .pl, .sh, .cgi, /shell invocations</td></tr>";
            R += "<tr><td>noNum</td><th>" + httpBounce.noProc + "</th><td>true</td><td>true</td><td>if true, rejects all IP Host requests (non DNS)</td></tr>";
            R += "</table><br>Note : you may use an API based IPOrigin resolver, the default whois takes over in case of error<br>Simply add the optional core.Items (or custom ones)<br><textarea cols='100' rows='6' class='bt'>    \"IPO\": {\n" +
"        \"Class\": \"ca.planj.bounce.origin.BigDataCloud\",\n" +
"        \"API\": \"https://api.bigdatacloud.net/data/ip-geolocation?ip=[IP]&localityLanguage=en&key=[key]\",\n" +
"        \"Key\": \"<!$BDCAPI_key$!>\"\n   },\n</textarea><br>for the BigDataCloud resolver or<br><textarea cols='100' rows='6' class='bt'>    \"IPO\": {\n" +
"        \"Class\": \"ca.planj.bounce.origin.IPRegistry\",\n" +
"        \"API\": \"https://api.ipregistry.co/[IP]?key=[key]\",\n" +
"        \"Key\": \"<!$IPReg_key$!>\",\n" +
"    },\n</textarea><br>for the IPRegistry API - with your keys, to the PlanJBounce.json file and use the launch= parameter<br>Note that resolver sources are OpenSource in the PJBSources.zip file";
            R += "<br><br>To add SSL/TLS on the Status server, you need an existing 'jks' file with your certificate (should match the FQDN)<br>- add this core.Item configuration before Status<br><textarea cols='100' rows='5' class='bt'>" +
                    "  \"myTLS\": {\n        \"Class\": \"ca.planj.crypto.pjSSLContext\",\n" +
"        \"*\": [ \"Path_To_Your_JKS\", \"PWD_for_JKS\" ]\n    },\n" +
                    "</textarea><br>Then add the matching line in the Status section : \"sslContext\": \"myTLS\",";
            R += "</td><tr></table>";
            R += "</body></html>";
            
            // Sending back to client
            if (R != null) {
                H.Link.write(H.getHeader(200, R.length(), "text/html", null));
                H.Link.write(R);
                if (debug) System.out.println("Sent " + H.Link);
                return true;
            }
        }
        return false;
    }
 int LLL = -1;
}
