/*
 * PlanJBounce components are Open Source
 * PlanJBounce requires PlanJCore (not Open Source)

2023-10-26 Fixed null test on GET
2023-10-23 Full rebuild and externalize to PlanJBounce Project from June 2022 prototype
    No imports, fully explicit object names
2023-10-01 Adding noNum Parameter, defering isListening to Handler (hung threads)
   no /xxx = Bad URI
2023-10-02 Adding a Timertask to Force Close connections after processing, Chrome was hanging, Firefox was not when using an IP based request
    Tuning done() vs Overflow attempts
    Prep modification for Bouncer v2 Object
2023-10-03 added tailored result codes, waiting for overflow attack, keeping trace info on console (start, URI, Host, Done)
   Added port stripping on IP Host check ip:port fails numeric conversion
   Cleanup had lost initial Naming
   Adding Stripped count
2023-10-04 Reinstating done() call at entry point giving a max life of 150 ms to Provider Threads - some connections hanging in
    Cleaning at exit might miss if nothing get processed (lingering TCP connection with no data)
    Revised cnt / iter handling between name changes
    Implemented No Host - No URI as TCP Ping no answer even with Polite - OK tested
2023-10-04 Moved connect count to st loop, drop track of TCP Ping
    Pretty stable, occasional whois query read errors, Linux and Mac sources vary samples : 181.41.206.226
2023-10-09 change Bad URI message, adding .env to bad uri
2023-10-11 Host port stripping earlier, better handling of redirects
2023-10-15 Added Overloop Messages and no Host:URI for extra visibility. Stable, Origin needs tweaking - lacks precision
2023-10-16 Adding Bad Method tracking info and double eMess
2023-10-20 Minor adjustments to Origin etc
2023-10-23 Change 403 to 404 on executable, avoids IDS detection
2023-10-23 0.3.0 Handling User-Agent in Bouncer
 */
package ca.planj.bounce;

/**
 * This is the PlanJCore tcpServer Handler module for PlanJBounce
 * It is a simplified variant of a full httpServer designed only to redirect or intercept requests
 * The initial goal is to have a forced redirection to TLS that does not expose the main Web server
 * It also rejects unwanted requests based on configurable flags
 * @author jipi@planj.ca
 */
public class httpBounce extends ca.planj.com.Handler {
    public static String Version = "Plan-J HTTP Bounce Server 0.3.1 2023-10-26";
    public static boolean debug = false;
    
    /**
     * Required to instantiate Handlers
     * @param N - Component Name
     * @return 
     */
    public static ca.planj.com.Handler getHandler(String N) {
        return new httpBounce(N);
    }
    
    static int cnt = 0;
    public httpBounce() {}
    public httpBounce(String N) {
        iter = cnt++;
        Name = N + "_" + iter;
    }
    
    /**
     * - no running Thread, just a fast and dirty inline process(); launched by the Link - remove this to go threaded
     * */
    @Override
    public void start() {
        process();
    }

    // Static configuration variables and default values
    static String Prefix = null;
    static String Suffix = null;
    
    static boolean Polite = true;
    static boolean Get = true;
    static boolean Strip = true;
    static boolean noProc = true;
    static boolean noNum = true;
    static boolean Track = false;
    
    // Instance counter, nice to have
    int iter = 0;     
    
    /**
     * Standard core.Item init(data.Node) portion
     * We capture static values, processed only once
     * @param N
     * @return 
     */
    public boolean init(ca.planj.data.Node N) {
        super.baseInit(N);
        Name = "PJBounce_" + iter;
        // Save a little tim if we are not the first one initialized
        if (Prefix != null) return true;
        
        // Analyze the pattern String
        String Patt = (String) N.getValue("Pattern");
        if (Patt != null) {
            int a = Patt.toLowerCase().indexOf("[host]");
            if (a > -1) try {
                Prefix = Patt.substring(0,a);
                Suffix = Patt.substring(a+6);
            } catch (Exception y) {}
        }
        //Apply default pattern http -> https if none specified
        if (Prefix == null) Prefix = "https://";
        if (Suffix == null) Suffix = "";
        
        //  Strict = all restrictions GET = only Get  Strip = No Params  noProc = reject .asp/.jsp/.php etc   polite = send 404 message
        String Mode = (String) N.getValue("Mode");
        if (Mode != null) {
            Mode = Mode.toLowerCase();
            if (Mode.contains("strict")) {
                Polite = false;
                Get = true;
                Strip = true;
                noProc = true;
                noNum = true;
            } else {
                Polite = Mode.contains("polite");
                Get = Mode.contains("get");
                Strip = Mode.contains("strip");
                noProc = Mode.contains("noproc");
                noNum = Mode.contains("nonum");
            }
            Track = Mode.contains("track");
            debug = Mode.contains("debug");
        }
        if (debug) {
            Track = true;
            Bouncer.debug = true;
            BounceView.debug = true;
        }
        if (debug) 
            System.out.println("httpBounce Config " + Prefix + " - " + Suffix + " " + Polite + " " + Get + " " + Strip + " " + noProc + " " + noNum);
        // Not really used
        Bouncer.setCurrent(Prefix, Suffix, Polite, Get, Strip, noProc);
        return true;
    }
    
    long st = 0;                      // Start Time, potential timeout usage
    int p = 0;                        // Process loops, kill if it spins out
    String rCode = "404 Not Found";   // default response
    
    /**
     * Standarad Handler Process Loop, using the Link Object
     * @return 
     */
    public boolean process() {
        String URI = null;
        String Meth = null;
        String Host = null;
        String Params = null;
        String redir = null;
        String Remote = (String) Link.getProvider().getProperty("remoteaddress");
        Bouncer B = Bouncer.getFor(Remote);  // Get the new Bouncer Object
        if (st == 0) {
            B.Count++; // Moved in st loop
            st = System.currentTimeMillis();
            if (Track)
                System.out.println("Just Started " + Name + " for " + Remote + " on " + Link.Name + " / " + Link.getProvider().Name);
            done();
        } else {
            if ((System.currentTimeMillis() - st) > 15000) {
                if (debug) 
                    System.out.println("Got Overtime on " + Name);
                Link.cleanup();
                return false;
            }
        }
        
        String eMess = null; // error message
        if (debug)
            System.out.println("Bouncer2 Connect from " + Remote + " -- " + Name);
        String L = null;
        String R = null;
        long limit = System.currentTimeMillis() + 5000;
        boolean isok = false;
        java.util.Vector Req = new java.util.Vector(15,10);
        // boolean active = true;
        boolean PO = Polite;
        ca.planj.ip.util.IPOrigin.fetchFor(Remote); // - 2023-10-19 see if it is redundant, indirect call when getting Bouncer, still lags on 2023-10-21 without this
        while ((L = Link.readLine()) != null) try {
            // Overflow protection attempt 2023-10-02 15:00 had 1000+ req from 103.252.136.176 with no Hostname for /
            if (p > 20) {
                // reaching up to 10 headers
                if (Track)
                    System.out.println(Name + " Reached 20");
                if (TT != null) {
                    B.add404("Overloop Header : " + Host + ":" + URI);
                    return false;
                } else {
                    B.add404("Overloop URI : " + Host + ":" + URI);
                }
                done();
            }
            // We collect the lines
            Req.addElement(L);
            p++;
            if (debug) System.out.println(p + " " + L);
            if (L == null) {
                // Trapping a rare overrun condition of early versions
                if (!Link.isConnected()) {
                    if (debug) System.out.println("Link Disconnect");
                }
                Thread.currentThread().sleep(2);
            } else {
                if (L.contains(" HTTP/")) {
                    int a = L.indexOf(" ");
                    int c = L.lastIndexOf(" ");
                    URI = L.substring(a+1,c);
                    // 414 URI Too Long 
                    rCode = "414 URI Too Long";
                    Meth = L.substring(0,a).trim();
                    eMess = null;
                    R = null;
                    if (Get) if (Meth != null) if (!Meth.equals("GET")) {
                        // 405 Method not allowed
                        isok = false;
                        eMess = Meth + " not allowed";
                        rCode = "405 Method not allowed";
                    }
                    isok = true;
                    if ((!URI.startsWith("/")) || URI.startsWith("//")) {
                        //400 Bad Request
                        rCode = "400 Bad Request";
                        if (eMess != null) eMess = eMess + ", Bad URI";
                        else eMess = "Bad URI";
                        isok = false;
                    } else if (Track)
                        System.out.println(Name + " -> " + URI);
                    // I should LOG the URI here
                } else if (L.startsWith("User-Agent: ")) {
                    B.setUserAgent(L.substring(12).trim());
                } else if (L.startsWith("Host: ")) {
                    Host = L.substring(6).trim();
                    if (Host.contains(":")) Host = Host.substring(0,Host.indexOf(":")); // Port fix 2023-10-11
                    if (Track)
                        System.out.println(Name + " --> " + Host);
                    if (noNum) try {
                        String Z = Host.substring(Host.lastIndexOf(".")+1);
                        //if (Z.contains(":")) Z = Z.substring(0,Z.indexOf(":")); // 2023-10-03 Port indicator impacts IP detection 
                        int F = Integer.parseInt(Z);
                        isok = false;
                        if (eMess != null) eMess = eMess + ", Host FQDN required";
                        else eMess = "Host FQDN required";
                        // 400 Bad Request
                        rCode = "400 Bad Request";
                    } catch (Exception u) {}
                    if (Strip) if (URI.contains("?")) {
                        // Stripping parameters
                        URI = URI.substring(0,URI.indexOf("?"));
                        B.Stripped++;
                    }
                    if (noProc) {
                        String uri = URI.toLowerCase();
                        if (debug) System.out.println("Testing noProc : " + uri);
                        isok = !(uri.contains(".asp")|| uri.contains(".jsp") || uri.contains(".pl") || uri.contains(".php") || uri.contains(".cgi") || uri.contains(".sh") || uri.contains("/shell") || uri.contains("..") || uri.contains(".env"));
                        if (!isok) {
                            if (eMess != null) eMess = eMess + ", Executable rejected";
                            else eMess = "Executable rejected";
                            rCode = "404 Not Found"; //"403 Forbidden";
                        }
                    }
                    if (isok) if (eMess == null) {
                        if (URI == null) URI = "/InVaLid";  // This replacement should never happen
                        URI = URI.trim();
                        // Might use 307 instead...
                        R = "HTTP/1.1 301 Moved Permanently\r\nLocation: " + Prefix + Host + Suffix + URI + "\r\nServer: " + Version + "\r\n\r\n";
                        if (URI.length() > 55) URI = URI.substring(0,50) + "(...)";
                        B.add302(Host + " " + URI);
                        Link.write(R);
                        if (debug) System.out.println("Got Host " + eMess + "-" + R);
                        // Most Frequesnt on OK
                        return done();
                    }
                } else 
                    if (L.trim().length() < 1) {
                        if (debug) System.out.println("Empty Line");
                        //active = false;
                        break;
                    }
            }
            if (System.currentTimeMillis() > limit) {
                if (debug) System.out.println("Timeout");
                break; //active = false;
            }
        } catch (Exception y) {  // End of Read Loop ***********
            if (debug) System.out.println("Exc " + y);
        }
        if (!isok) if (p > 0) if (eMess == null) {
            eMess = "No Host";
            rCode = "400 Bad Request";
            PO = false;
            if (URI == null) {
                // TCP Ping / Test was received - no reply even if Polite, tested OK
                // Link.cleanup();
                //B.add404("TCP empty : No Host/URI");
                return false;
            }
        }
        if (eMess != null) {
            // This is where we respond on Errors / Rejected requests
            if (URI == null) URI = "No URI"; // If there was no URI, we set this variant of Invalid
            URI = URI.trim();
            if (URI.length() > 50) URI = URI.substring(0,30) + "(...)";
            B.add404(eMess + " : " + Host + " : " + URI);
            String K = "HTTP/1.1 " + rCode + "\r\nServer: " + Version + "\r\nContent-Type: text/plain\r\nContent-Length: " + (rCode.length()+eMess.length()+8) + "\r\n\r\nError " + rCode + ". " + eMess;
            if (PO) {
                if (debug) System.out.println("Sending : " + K);
                Link.write(K);
                return false;
            }
            eMess = null;
            R = null;
            // Most Frequent on error
            return done();
        } else if (R != null) {
            if (URI == null) URI = "/InvaLiD";
            URI = URI.trim();
            if (URI.length() > 50) URI = URI.substring(0,30) + "(...)";
            B.add302(Host + " : " + URI);
            try { Link.write(R); } catch (Exception e) {
                if (debug) System.out.println(Remote + " " + Name + " had closed");
            }
            // Should be never
            return done();
        } else try { Thread.sleep(10); } catch (Exception z) { if (debug) System.out.println("*** " + z); }
        if (debug) System.out.println("EOP " + Polite + " " + isok + " " + p + " " + R + " -- " + eMess + " " + URI);
        R = null;
        // If nothing was received we will loop
        return (eMess == null);
    }
    
    /**
     * 2023-10-03 Might be overkill with new processes / direct cleanup needs to be tested 
     * Connection ending section, to get rid of sticky HTTP/1.1 or other persistent connection
     * We redirect and flush
     */
    private class blurb extends java.util.TimerTask {
        httpBounce X = null;
        public void run() {
            if (X != null) try {
                X.Link.cleanup();
                X.Link.closeProvider();
            } catch (Exception y) {}
        }
    }
    blurb TT = null;
    /**
     * Called when a reply is sent, Timer Task to prevent timing delays during asynchronous send
     * @return 
     */
    private boolean done() {
        if (Track) {
            StackTraceElement[] T = Thread.currentThread().getStackTrace();
            System.out.println("Calling DONE for " + Name + " at " + p + " with " + TT + " from " + T[2]);
        }
        // 2023-10-04 fast close / breaks
        //Link.cleanup();
        //if (true) return false;
        
        // Delayed close
        if (TT == null) try {
            TT = new blurb();
            TT.X = this;
            java.util.Timer TX = new java.util.Timer(true);
            TX.schedule(TT, 250);  // Basically a forced timeout setting
        } catch (Exception y) {}
        return false;
    }
    
}
