/*
 * PlanJBounce components are Open Source
 * PlanJBounce requires PlanJCore (not Open Source)

 * Bounce server for port 80 - eliminates some attacks and redirects to https or else
2023-09-24 initiate full PlanJBounce review
2023-09-26/27 1.0.2 to 1.0.20 debug on live Internet Connection, added origin (14) (should go in Core when stable) added reverseDNS (18) custom load (19)
2023-10-01 adding noNum, will have to check into unreleased client threads, kill them after 5 seconds - they are redirected
    Multiple tuning fixed, performance, isListening, synchronized addGood/Bad
2023-10-03 Tuning Origin, duplicates not showing, still hanging threads - testable via Chrome... (Firefox closes)
    1.1.0 Full revision of Bouncer and BounceView
2023-10-03 1.1.14 Very stable, minor adjustments on BView, no overflow attempts overnight (main checkup point to validate)
    Will need to return varied response codes
2023-10-04 Retuning robustness, 150 calls from india left 1 thread hanging
    1.2.14 small fixes, .13 was stable
2023-10-05 Minor tracking adjustments, occasional origin whois misread on Linux
2023-10-06 Stable, prep for BounceView Filters
2023-10-10 1.3+ Filtering in BounceView, nicely came through a 800+ request probe
2023-10-11 1.4+ Added /BNode (json) and /BNode?purge, reviewed Origin, improved Host port handling
        1.4.1 adds a NetACL on the management interface
2023-10-12 Minor bugs on Origin, reset version to 1.0
2023-10-13 origin modified to include pre parsed Owner and Country info + fetchOrigin() object vs fetchFor() Description
2023-10-15 More info on bad calls etc, stable - pre initialize Origin - refine origin, fix filter issue
2023-10-17 Null Country occasional crash, increase robustness (PJStatus Error Log is nice to find exact issue) - 1.0.8 had a strange BView slowdown on Origin (I Think) - PJStatus was still OK
2023-10-18 adapt to IPOrigin (1.0.11 contains JSParse2, extra IPOrigin details etc. - to be deleted in 1.0.12)
2023-10-20 Tuning resolvers, had a slowdown
2023-10-22 1.0.16 added getDNS to async thread - delay at BView load, 1.0.17 - swap to BigDataCloud Origins - 1.0.18 refetch Origin on Processing - seems OK
    1.1.0 Default Config written to File and better Instructions on BView
    Unkown crash on BNode while formatting, added trace in JsonIO.toJSON to see cause in output
2023-10-23 1.1.1 another fix on getOrigin, 1.1.4 Autoload PlanJBounce.json if it exists 1.1.5 fix on JSON Length and BView display, 1.1.6 404 instead of 403 on exec
    1.2.0 Includes User-Agent analysis / reporting, adding text for TLS configuration
2023-10-24 1.2.1 Reaching in the 20Kb of JSON data, BView loses a closing } again. Tryng to convert to byte[] early
2023-10-25 1.2.2 includes Bouncer.Ingest(Node) to reimport BView data, reparse UserAgent
    1.2.3 Confirmed byte[] for BNode better 2023-10-24 17:06:15 INFO null BView Anomaly9390 9388 - why is the source String shorter ... UTF-8 char ?
2023-10-26 1.2.4 Get Null test was bad, validated via unifi
2023-10-29 1.2.5 Minor changes related to PlanJCore review - will require Time based reporting and persistence (lists are getting large)
Future : Complete redirection map based on origin, with 302 Moved Temporarily to redistribute resources, instead of 301, or 307 more recent way
 */
package planjbounce;

/**
 * This is the Main Class of the PlanJBounce Package
 * It is preconfigured for autonomous execution with PlanJCore but without CoreBoot
 * By default it launches a redirector from port 8786 to https://[original URL] if the request is correctly formatted
 *     Typically set Router Port 80 Forwarding to this system's port 8786 and Port 443 to the main https server
 * The Management URI is on Port 8787 at /BView http://localhost:8787/BView
 *     The PlanJCore Status Page is active at http://localhost:8787/PJStatus
 *     There is no security on the management pages, except for a LAN only ACL
 *     Custom Configuration instructions are on the /BView Page
 *     Data Export available (json) on /BNode
 * @author jipi@planj.ca
 */
public class PlanJBounce {
    public static String Version = "PlanJBounce 1.2.5 2023-10-29";
    public static String[] RequiredLibs = { "PlanJCore" };
    public static String[][] RequiredItems = { 
        { "ca.planj.ip.tcpServer",  "ca.planj.ip.tcpServer"},  // Required core.Items
        {  }   // Optional core.Items
    };
    /**
     * Direct Launch (recommended) optional load=[json file], reference in :8787/BView
     * Uses boot.Identify for minimal info (not a full CoreBoot)
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        long t = System.currentTimeMillis();
        System.out.println(Version + " using " + ca.planj.PlanJCore.Version);
        // Need AppID defined
        ca.planj.data.Node Boot = ca.planj.data.Node.makeDataNode("BootInfo");
        ca.planj.core.monit.SysMonitor.setBootInfo(Boot);
        Boot.addChild("AppID","PJBouncer");                     // Will display in PJStatus
        Boot.addChild("Package","PlanJBounce");                 // For reference only
        Boot.addChild("PlanJCore",ca.planj.PlanJCore.Version);  // Just to be sure it is available
        ca.planj.boot.Identify.whatAmI(args);                   // Minimal Info for /PJStatus and updater
        
        java.io.File AppF = new java.io.File("PlanJBounce.json");
        //ca.planj.data.Node N = Boot.addChild("Path",null); 
        Boot.addChild("AppFile",AppF.getAbsolutePath());
        // Write Default or read actual
        if (!AppF.exists()) ca.planj.util.QuickRead.writeFile(AppF.getAbsolutePath(), Bounce);
        else Bounce = ca.planj.util.QuickRead.readFile(AppF.getAbsolutePath());
        //ca.planj.core.monit.SysMonitor.debug = true;
        //ca.planj.http.Status.debug = true;
        
        // set a load param for custom
        if (args != null) for (int i=0; i<args.length; i++) if (args[i].toLowerCase().startsWith("load=")) {
            String txt = ca.planj.util.QuickRead.readFile(args[i].substring(5));
            if ((txt != null) && (txt.length() > 20)) {
                Bounce = txt;
            } else {
                System.out.println("Invalid configuration file at " + args[i].substring(5));
            }
        }
        ca.planj.boot.Configure.loadJSon(Bounce);
        ca.planj.ip.util.IPOrigin.fetchFor("127.0.0.1");
        Boot.addChild("BootDuration",(System.currentTimeMillis()-t));  // 
        
        if (false) {
            // Ingest - not for prod, bring in data 
            String txt = ca.planj.util.QuickRead.readFile("/Users/jipi/SDrive/NBSync/PlanJCore/test/dta.json");
            System.out.println(txt.length());
            ca.planj.data.Node[] Y = ca.planj.data.nodeio.JSonIO.getInstance().ingest(txt);
            System.out.println(Y.length);
            String[] U = Y[1].getChildNames();
            for (int i=0; i<U.length; i++) {
                String IP = ca.planj.util.StringUtils.ReplaceStr(U[i], "_", ".");
                ca.planj.data.Node W = Y[1].getChild(U[i]);
                ca.planj.bounce.Bouncer.Ingest(W);
            }
            System.out.println("Ingested " );
        }
    }
   
    /**
     * Built in Default Configuration (PlainText for copy-paste in http://localhost:8787/BView )
     * Can be externalized, modified and used by adding load=[path to file] on the command line
     *    java -jar PlanJBounce.jar load=[your_file_path_and_name]
     */
    public static String Bounce = "{\n    \"Bounce\": {\n" +
"        \"Class\": \"ca.planj.ip.tcpServer\",\n" +
"        \"Handler\": \"ca.planj.bounce.httpBounce\",\n" +
"        \"Pattern\": \"https://[Host]\",\n" + //  https://[Host]:8443/SubPath
"        \"Mode\": \"NoProc Get Strip noNum Polite\",\n" +  // Track
"        \"Port\": 8786\n" +
"    },\n" +
"    \"LAN_ACL\": {\n" +
"        \"Class\": \"ca.planj.ip.util.NetACL\",\n" +
"        \"DefaultAllow\": false,\n" +
"        \"Allow\": [ \"10.0.0.0/8\", \"172.16.0.0/12\", \"192.168.0.0/16\", \"127.0.0.1\" ]\n" +
"    },\n" +
"    \"Status\": {\n" +
"        \"Class\": \"ca.planj.ip.tcpServer\",\n" +
"        \"Handler\": \"ca.planj.com.handle.httpServer\",\n" +
"        \"Modules\": [ \"ca.planj.bounce.BounceView\" ],\n" +
"        \"Port\": 8787,\n" +
"        \"netACL\": \"LAN_ACL\",\n" +
"        \"SiteIcon\": \"/ca/planj/http/pjBicon.png\",\n" +
"        \"StatusPage\": \"/PJStatus\"\n" +
"    }\n" +
"}";
}
